﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
//ADD
using System.Data;
using System.Data.SqlClient;

public partial class Cart : System.Web.UI.Page
{
    //ページ読込時の処理
    protected void Page_Load(object sender, EventArgs e)
    {
        if(!IsPostBack)
		{
            //データベース接続
            string conString = System.Configuration.ConfigurationManager
                                .ConnectionStrings["sampleConnectionString"].ConnectionString;
            SqlConnection cn = new SqlConnection(conString);
            cn.Open();

            string sql = "select * from [Customer] where [CustomerNo] = @CustomerNo";
            SqlCommand cm = new SqlCommand(sql, cn);
            cm.Parameters.Add("CustomerNo", Session["CustomerNo"].ToString());
            SqlDataReader dr = null;
            try
            {
                dr = cm.ExecuteReader();
                if (dr.Read())
                {
                    //顧客情報の初期表示
                    txtName.Text        = dr["Name"].ToString();
                    txtPhoneNumber.Text = dr["PhoneNumber"].ToString();
                    txtPostalCode.Text  = dr["PostalCode"].ToString();
                    txtPrefecture.Text  = dr["Prefecture"].ToString();
                    txtCity.Text        = dr["City"].ToString();
                    txtAddress1.Text    = dr["Address1"].ToString();
                    txtAddress2.Text    = dr["Address2"].ToString();
                    txtPhoneNumber.Text = dr["PhoneNumber"].ToString();
                    txtEmail.Text       = dr["Email"].ToString();
                }
            }
            finally
            {
                dr.Close();
                cn.Close();
            }

            //カート情報の更新
            DataTable cart = (DataTable)Session["Cart"];
            UpdateCart();
        }
    }

    //カート情報の更新
    private void UpdateCart()
    {
        DataTable cart = (DataTable)Session["Cart"];

        //GridViewの更新
        grdCart.DataSource = cart.DefaultView;
        grdCart.DataBind();

        //合計金額の更新
        int price = 0;
        foreach (DataRow dr in cart.Rows)
        {
            price += (int)dr["UnitPrice"] * (int)dr["OrderQuantity"];
        }

        //送料の計算
        int ship;
        if (price >= 5000)
        {
            //合計が5000円以上なら無料とする
            ship = 0;
        }
        else
        {
            //5000円未満の場合は一律500円とする
            ship = 500;
        }
        price += ship;

        lblTotalPrice.Text = price.ToString();
        lblShippingCharge.Text = ship.ToString();
    }

    //「数量変更」ボタン・「削除」ボタンの処理
    protected void grdCart_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        DataTable cart = (DataTable)Session["Cart"];

        //数量変更
        if (e.CommandName == "Change")
        {
            int index = Int32.Parse((String)e.CommandArgument);

            DataRow[] drs = cart.Select("ProductNo = '" + grdCart.Rows[index].Cells[0].Text + "'");
            if (drs.Length > 0)
            {

                DataRow dr = drs[0];
                dr["OrderQuantity"] = ((TextBox)grdCart.Rows[index].Cells[2].FindControl("txtOrderQuantity")).Text;
                dr["Price"] = ((int)dr["UnitPrice"] * (int)dr["OrderQuantity"]).ToString();
            }
        }
        //削除
        else if (e.CommandName == "Del")
        {
            int index = Int32.Parse((String)e.CommandArgument);
            cart.Rows[index].Delete();
        }
        Session["Cart"] = cart;
        UpdateCart();
    }
    
    //「注文を確定する」ボタンの処理
    protected void btnOrder_Click(object sender, EventArgs e)
    {
        //データベース接続
        string conString = System.Configuration.ConfigurationManager
                            .ConnectionStrings["SampleConnectionString"].ConnectionString;
        SqlConnection cn = new SqlConnection(conString);
        cn.Open();
        
        //トランザクションの開始
        SqlTransaction tr = cn.BeginTransaction();
        SqlDataReader dr = null;
        int OrderNo;

        try
        {
            //注文番号の取得
            string sql = "select max([OrderNo]) + 1 as [OrderNo] from [Order]";
            SqlCommand cm = new SqlCommand(sql, cn, tr);
            dr = cm.ExecuteReader();
            try
            {
                dr.Read();
                //最大の注文番号+1とする
                OrderNo = Int32.Parse(dr["OrderNo"].ToString());
            }
            catch
            {
                OrderNo = 1; //データがない場合は1とする
            }
            dr.Close();

            //OrderテーブルのINSERT
            sql = "insert into [Order](";
            sql += "[OrderNo],";
            sql += "[OrderDatetime],";
            sql += "[CustomerNo],";
            sql += "[ShippingCharge])";
            sql += "values ";
            sql += "(@OrderNo,";
            sql += "@OrderDatetime,";
            sql += "@CustomerNo,";
            sql += "@ShippingCharge)";
            cm.CommandText = sql;
            cm.Parameters.Clear();
            cm.Parameters.Add(new SqlParameter("OrderNo", OrderNo));
            cm.Parameters.Add(new SqlParameter("OrderDatetime", DateTime.Today.ToString("yyyy/MM/dd")));
            cm.Parameters.Add(new SqlParameter("CustomerNo", Session["CustomerNo"].ToString()));
            cm.Parameters.Add(new SqlParameter("ShippingCharge", lblShippingCharge.Text));
            cm.ExecuteNonQuery();

            //OrderDetailテーブルのINSERT
            sql = "insert into [OrderDetail](";
            sql += "[OrderNo],";
            sql += "[ProductNo],";
            sql += "[OrderQuantity])";
            sql += " values ";
            sql += "(@OrderNo,";
            sql += "@ProductNo,";
            sql += "@OrderQuantity)";
            cm.CommandText = sql;

            DataTable cart = (DataTable)Session["Cart"];
            foreach (DataRow item in cart.Rows)
            {
                cm.Parameters.Clear();
                cm.Parameters.Add(new SqlParameter("OrderNo", OrderNo));
                cm.Parameters.Add(new SqlParameter("ProductNo", item["ProductNo"].ToString()));
                cm.Parameters.Add(new SqlParameter("OrderQuantity", item["OrderQuantity"].ToString())); 
                cm.ExecuteNonQuery();
            }

            //CustomerテーブルのUPDATE
            sql = "update [Customer] set ";
            sql += "Name = @Name, ";
            sql += "[PostalCode] = @PostalCode, ";
            sql += "[Prefecture] = @Prefecture, ";
            sql += "[City] = @City, ";
            sql += "[Address1] = @Address1, ";
            sql += "[Address2] = @Address2, ";
            sql += "[PhoneNumber] = @PhoneNumber, ";
            sql += "[Email] = @Email ";
            sql += "where [CustomerNo] = @CustomerNo";
            cm.CommandText = sql;
            cm.Parameters.Clear();
            cm.Parameters.Add(new SqlParameter("Name", txtName.Text));
            cm.Parameters.Add(new SqlParameter("PostalCode", txtPostalCode.Text));
            cm.Parameters.Add(new SqlParameter("Prefecture", txtPrefecture.Text));
            cm.Parameters.Add(new SqlParameter("City", txtCity.Text));
            cm.Parameters.Add(new SqlParameter("Address1", txtAddress1.Text));
            cm.Parameters.Add(new SqlParameter("Address2", txtAddress2.Text));
            cm.Parameters.Add(new SqlParameter("PhoneNumber", txtPhoneNumber.Text));
            cm.Parameters.Add(new SqlParameter("Email", txtEmail.Text));
            cm.Parameters.Add(new SqlParameter("CustomerNo", Session["CustomerNo"].ToString()));
            cm.ExecuteNonQuery();

            //ProductテーブルのUPDATE
            sql = "update [Product] set ";
            sql += "[StockQuantity] = [StockQuantity] - @OrderQuantity ";
            sql += "where [ProductNo] = @ProductNo";
            cm.CommandText = sql;
            foreach (DataRow item in cart.Rows)
            {
                cm.Parameters.Clear();
                cm.Parameters.Add(new SqlParameter("OrderQuantity", item["OrderQuantity"].ToString()));
                cm.Parameters.Add(new SqlParameter("ProductNo", item["ProductNo"].ToString()));
                cm.ExecuteNonQuery();
            }

            //トランザクションをコミットする
            tr.Commit();
        }
        catch(Exception Ex)
        {
            //例外発生時はトランザクションをロールバックする
            tr.Rollback();

            //例外のエラーメッセージを画面に表示
            Response.Write(Ex.Message);
            return;
        }
        finally
        {
            dr.Close();
            cn.Close();
        }

        Session["OrderNo"] = OrderNo;
        Response.Redirect("Finish.aspx");
    }

    //「商品一覧に戻る」ボタンの処理
    protected void btnBack_Click(object sender, EventArgs e)
    {
        Session["CustomerNo"] = null;
        Session["Cart"] = null;
        Response.Redirect("Product.aspx");
    }

}
